import json
import logging

from .model import (
    APIModel,
    APIEndpoints,
    RequestsMethods,
)
from .api import Api


class Reporting:
    """The class includes all necessary methods to access the Grafana reporting API endpoints. Be aware that the functionality is a Grafana ENTERPRISE v7.0+ feature

    HINT: Note Grafana Enterprise API need required permissions if fine-grained access control is enabled

    Args:
        grafana_api_model (APIModel): Inject a Grafana API model object that includes all necessary values and information

    Attributes:
        grafana_api_model (APIModel): This is where we store the grafana_api_model
    """

    def __init__(self, grafana_api_model: APIModel):
        self.grafana_api_model = grafana_api_model

    def send_report(
        self, id: int, emails: str = None, use_emails_from_report: bool = None
    ):
        """The method includes a functionality to send a report to a specified email addresses

        Args:
            id (int): Specify the id of sented report. It is the same as in the URL when editing a report, not to be confused with the id of the dashboard.
            emails (str): Specify the comma-separated list of emails to which to send the report to. Overrides the emails from the report. Required if useEmailsFromReport is not present (default None)
            use_emails_from_report (bool): Specify the if the emails inside the report should be used. Required if emails is not present (default None)

        Required Permissions:
            Action: reports:send
            Scope: N/A

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if id != 0 and (
            emails is not None
            and (len(emails) != 0)
            or (use_emails_from_report is not None and use_emails_from_report)
        ):
            if emails is not None and len(emails) != 0:
                result: dict = dict({"id": id, "emails": emails})
            else:
                result: dict = dict(
                    {"id": id, "useEmailsFromReport": use_emails_from_report}
                )

            api_call: dict = (
                Api(self.grafana_api_model)
                .call_the_api(
                    APIEndpoints.REPORTING.value,
                    RequestsMethods.POST,
                    json.dumps(result),
                    timeout=60,
                )
                .json()
            )

            if api_call.get("message") != "Report was sent":
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                logging.info("You successfully send the report.")
        else:
            logging.error("There is no id, emails or useEmailsFromReport defined.")
            raise ValueError



import json
import logging

from .model import (
    APIModel,
    APIEndpoints,
    RequestsMethods,
)
from .api import Api


class QueryHistory:
    """The class includes all necessary methods to access the Grafana query history API endpoints. Be aware that it requires that the user is logged in and that query history feature is enabled in the config file

    Args:
        grafana_api_model (APIModel): Inject a Grafana API model object that includes all necessary values and information

    Attributes:
        grafana_api_model (APIModel): This is where we store the grafana_api_model
    """

    def __init__(self, grafana_api_model: APIModel):
        self.grafana_api_model = grafana_api_model

    def add_query_to_history(self, datasource_uid: str, queries: list) -> dict:
        """The method includes a functionality to add queries to query history

        Args:
            datasource_uid (str): Specify the datasource uid
            queries (list): Specify the queries as list from type QueryObject

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns the added result of the query history
        """

        if len(datasource_uid) != 0 and len(queries) != 0:
            queries_json_list: list = list()

            for query in queries:
                query_json_dict: dict = dict(
                    {
                        "refId": query.ref_id,
                        "key": query.key,
                        "scenarioId": query.scenario_id,
                        "datasource": dict(
                            {"type": query.datasource.type, "uid": query.datasource.uid}
                        ),
                    }
                )
                queries_json_list.append(query_json_dict)

            api_call: dict = (
                Api(self.grafana_api_model)
                .call_the_api(
                    APIEndpoints.QUERY_HISTORY.value,
                    RequestsMethods.POST,
                    json.dumps(
                        dict(
                            {
                                "datasourceUid": datasource_uid,
                                "queries": queries_json_list,
                            }
                        )
                    ),
                )
                .json()
            )

            if api_call == dict() or api_call.get("result") is None:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                return api_call
        else:
            logging.error("There is no datasource_uid or queries defined.")
            raise ValueError

    def search_query_history(
        self,
        datasource_uids: list,
        search_string: str,
        sort: str = "time-desc",
        only_starred: bool = False,
        pages: int = 1,
        results_per_page: int = 100,
    ) -> dict:
        """The method includes a functionality to search a query inside the query history

        Args:
            datasource_uids (list): Specify the datasource uid
            search_string (str): Specify the search string to filter the result
            sort (str): Specify the sorting order e.g. time-asc or time-desc (default time-desc)
            only_starred (bool): Specify if queries that are starred should be used for the search (default false)
            pages (int): Specify the pages as integer (default 1)
            results_per_page (int): Specify the results_per_page as integer (default 100)

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns the corresponding result of the query history
        """

        if len(datasource_uids) != 0 and len(search_string) != 0:
            datasource_uids_str: str = ""

            for i in range(0, len(datasource_uids)):
                datasource_uids_str = (
                    f"{datasource_uids_str}datasourceUid='{datasource_uids[i]}'"
                )

                if len(datasource_uids) != 1 and i != len(datasource_uids) - 1:
                    datasource_uids_str = f"{datasource_uids_str}&"

            api_call: dict = (
                Api(self.grafana_api_model)
                .call_the_api(
                    f"{APIEndpoints.QUERY_HISTORY.value}?{datasource_uids_str}&searchString='{search_string}'&sort='{sort}'&onlyStarred={only_starred}&page={pages}&limit={results_per_page}",
                )
                .json()
            )

            if api_call == dict() or api_call.get("result") is None:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                return api_call
        else:
            logging.error("There is no datasource_uids or search_string defined.")
            raise ValueError

    def delete_query_history(self, uid: str):
        """The method includes a functionality to delete a query inside the query history

        Args:
            uid (str): Specify the uid of the query

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
             None
        """

        if len(uid) != 0:
            api_call: dict = (
                Api(self.grafana_api_model)
                .call_the_api(
                    f"{APIEndpoints.QUERY_HISTORY.value}/{uid}", RequestsMethods.DELETE
                )
                .json()
            )

            if api_call.get("message") != "Query deleted":
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                logging.info("You successfully deleted the query.")
        else:
            logging.error("There is no uid defined.")
            raise ValueError

    def update_query_history(self, uid: str, comment: str) -> dict:
        """The method includes a functionality to update a query inside the query history

        Args:
            uid (str): Specify the uid of the query
            comment (str): Specify the comment of the query

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns the modified result of the query history
        """

        if len(uid) != 0 and len(comment) != 0:
            api_call: dict = (
                Api(self.grafana_api_model)
                .call_the_api(
                    f"{APIEndpoints.QUERY_HISTORY.value}/{uid}",
                    RequestsMethods.PATCH,
                    json.dumps({"comment": comment}),
                )
                .json()
            )

            if api_call == dict() or api_call.get("result") is None:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                return api_call
        else:
            logging.error("There is no uid or comment defined.")
            raise ValueError

    def star_query_history(self, uid: str) -> dict:
        """The method includes a functionality to star a query inside the query history

        Args:
            uid (str): Specify the uid of the query

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns the corresponding stared query history
        """

        if len(uid) != 0:
            api_call: dict = (
                Api(self.grafana_api_model)
                .call_the_api(
                    f"{APIEndpoints.QUERY_HISTORY.value}/star/{uid}",
                    RequestsMethods.POST,
                    json.dumps({}),
                )
                .json()
            )

            if api_call == dict() or api_call.get("result") is None:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                return api_call
        else:
            logging.error("There is no uid defined.")
            raise ValueError

    def unstar_query_history(self, uid: str) -> dict:
        """The method includes a functionality to unstar a query inside the query history

        Args:
            uid (str): Specify the uid of the query

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns the corresponding unstared query history
        """

        if len(uid) != 0:
            api_call: dict = (
                Api(self.grafana_api_model)
                .call_the_api(
                    f"{APIEndpoints.QUERY_HISTORY.value}/star/{uid}",
                    RequestsMethods.DELETE,
                )
                .json()
            )

            if api_call == dict() or api_call.get("result") is None:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                return api_call
        else:
            logging.error("There is no uid defined.")
            raise ValueError



import logging

from .api import Api
from .model import APIModel, APIEndpoints


class Search:
    """The class includes all necessary methods to access the Grafana search API endpoints

    Args:
        grafana_api_model (APIModel): Inject a Grafana API model object that includes all necessary values and information

    Attributes:
        grafana_api_model (APIModel): This is where we store the grafana_api_model
    """

    def __init__(self, grafana_api_model: APIModel):
        self.grafana_api_model = grafana_api_model

    def search(self, search_query: str) -> list:
        """The method includes a functionality to execute a custom query

        Args:
            search_query (str): Specify the inserted query as string

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (list): Returns the list of query the results
        """

        if len(search_query) != 0:
            result: list = (
                Api(self.grafana_api_model)
                .call_the_api(f"{APIEndpoints.SEARCH.value}?{search_query}")
                .json()
            )
            if result == list():
                raise Exception
            else:
                return result
        else:
            logging.error("There is no search_query defined.")
            raise ValueError



import json
import logging

from .model import (
    APIModel,
    APIEndpoints,
    RequestsMethods,
    UserObject,
)
from .api import Api


class User:
    """The class includes all necessary methods to access the Grafana user API endpoints. Be aware that all functionalities inside the class only working with basic authentication (username and password) and that the authenticated user is a Grafana Admin.

    HINT: Note Grafana Enterprise API need required permissions if fine-grained access control is enabled

    Args:
        grafana_api_model (APIModel): Inject a Grafana API model object that includes all necessary values and information

    Attributes:
        grafana_api_model (APIModel): This is where we store the grafana_api_model
    """

    def __init__(self, grafana_api_model: APIModel):
        self.grafana_api_model = grafana_api_model

    def search_users(
        self,
        results_per_page: int = 1000,
        pages: int = 1,
        query: str = None,
    ) -> list:
        """The method includes a functionality to get all Grafana system users specified by the optional query and paging functionality

        Required Permissions:
            Action: users:read
            Scope: global.users:*

        Args:
            results_per_page (int): Specify the results_per_page as integer (default 1000)
            pages (int): Specify the pages as integer (default 1)
            query (str): Specify the query (default None)

        Raises:
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (list): Returns the list of Grafana users
        """

        api_request_url: str = (
            f"{APIEndpoints.USERS.value}/search?perpage={results_per_page}&page={pages}"
        )

        if query is not None and len(query) != 0:
            api_request_url: str = f"{api_request_url}&query={query}"

        api_call: list = (
            Api(self.grafana_api_model)
            .call_the_api(
                api_request_url,
            )
            .json()
        )

        if api_call == list() or api_call[0].get("id") is None:
            logging.error(f"Check the error: {api_call}.")
            raise Exception
        else:
            return api_call

    def get_user_by_id(self, id: int) -> dict:
        """The method includes a functionality to get a specific user by the id

        Required Permissions:
            Action: users:read
            Scope: users:*

        Args:
            id (int): Specify the id of the user

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns the user information
        """

        if id != 0:
            api_call: dict = (
                Api(self.grafana_api_model)
                .call_the_api(
                    f"{APIEndpoints.USERS.value}/{id}",
                )
                .json()
            )

            if api_call == dict() or api_call.get("id") is None:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                return api_call
        else:
            logging.error("There is no id defined.")
            raise ValueError

    def get_user_by_username_or_email(self, username_or_email: str) -> dict:
        """The method includes a functionality to get a specific user by the username_or_email

        Required Permissions:
            Action: users:read
            Scope: global.users:*

        Args:
            username_or_email (str): Specify the username_or_email of the user

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns the user information
        """

        if len(username_or_email) != 0:
            api_call: dict = (
                Api(self.grafana_api_model)
                .call_the_api(
                    f"{APIEndpoints.USERS.value}/lookup?loginOrEmail={username_or_email}",
                )
                .json()
            )

            if api_call == dict() or api_call.get("id") is None:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                return api_call
        else:
            logging.error("There is no username_or_email defined.")
            raise ValueError

    def update_user(self, id: int, user: UserObject):
        """The method includes a functionality to update the specified user

        Required Permissions:
            Action: users:write
            Scope: users:*

        Args:
            id (int): Specify the id of the user
            user (UserObject): Specify the used UserObject

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if id != 0 and user is not None:
            api_call: dict = (
                Api(self.grafana_api_model)
                .call_the_api(
                    f"{APIEndpoints.USERS.value}/{id}",
                    RequestsMethods.PUT,
                    json.dumps(
                        dict(
                            {
                                "email": user.email,
                                "name": user.name,
                                "login": user.login,
                                "theme": user.theme,
                            }
                        )
                    ),
                )
                .json()
            )

            if api_call.get("message") != "User updated":
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                logging.info("You successfully modified the user.")
        else:
            logging.error("There is no id or user defined.")
            raise ValueError

    def get_user_organizations(self, id: int) -> list:
        """The method includes a functionality to get the specified user organizations

        Required Permissions:
            Action: users:read
            Scope: users:*

        Args:
            id (int): Specify the id of the user

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (list): Returns a list of the user bound organizations
        """

        if id != 0:
            api_call: list = (
                Api(self.grafana_api_model)
                .call_the_api(
                    f"{APIEndpoints.USERS.value}/{id}/orgs",
                )
                .json()
            )

            if api_call == list() or api_call[0].get("orgId") is None:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                return api_call
        else:
            logging.error("There is no id defined.")
            raise ValueError

    def get_user_teams(self, id: int) -> list:
        """The method includes a functionality to get the specified user teams

        Required Permissions:
            Action: users.teams:read
            Scope: users:*

        Args:
            id (int): Specify the id of the user

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (list): Returns a list of the user bound teams
        """

        if id != 0:
            api_call: list = (
                Api(self.grafana_api_model)
                .call_the_api(
                    f"{APIEndpoints.USERS.value}/{id}/teams",
                )
                .json()
            )

            if api_call == list() or api_call[0].get("id") is None:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                return api_call
        else:
            logging.error("There is no id defined.")
            raise ValueError

    def switch_specific_user_context(self, user_id: int, org_id: int):
        """The method includes a functionality to switch the user context to the given organization

        Args:
            user_id (int): Specify the user_id
            org_id (int): Specify the org_id

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if user_id != 0 and org_id != 0:
            api_call: dict = (
                Api(self.grafana_api_model)
                .call_the_api(
                    f"{APIEndpoints.USERS.value}/{user_id}/using/{org_id}",
                    RequestsMethods.POST,
                    json.dumps(dict()),
                )
                .json()
            )

            if api_call.get("message") != "Active organization changed":
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                logging.info("You successfully updated active organization.")
        else:
            logging.error("There is no user_id or org_id defined.")
            raise ValueError


class CurrentUser:
    """The class includes all necessary methods to access the Grafana current user API endpoints. Be aware that all functionalities inside the class maybe only working with basic authentication (username and password)

    Args:
        grafana_api_model (APIModel): Inject a Grafana API model object that includes all necessary values and information

    Attributes:
        grafana_api_model (APIModel): This is where we store the grafana_api_model
    """

    def __init__(self, grafana_api_model: APIModel):
        self.grafana_api_model = grafana_api_model

    def get_user(self) -> dict:
        """The method includes a functionality to get the current user

        Raises:
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns the user information
        """

        api_call: dict = (
            Api(self.grafana_api_model)
            .call_the_api(
                f"{APIEndpoints.USER.value}",
            )
            .json()
        )

        if api_call == dict() or api_call.get("id") is None:
            logging.error(f"Check the error: {api_call}.")
            raise Exception
        else:
            return api_call

    def update_password(
        self, old_password: str, new_password: str, confirm_new_password: str
    ):
        """The method includes a functionality to update the current user password

        Args:
            old_password (str): Specify the old_password
            new_password (str): Specify the new_password
            confirm_new_password (str): Specify the confirm_new_password

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if (
            len(old_password) != 0
            and len(new_password) != 0
            and len(confirm_new_password) != 0
        ):
            api_call: dict = (
                Api(self.grafana_api_model)
                .call_the_api(
                    f"{APIEndpoints.USER.value}/password",
                    RequestsMethods.PUT,
                    json.dumps(
                        dict(
                            {
                                "oldPassword": old_password,
                                "newPassword": new_password,
                                "confirmNew": confirm_new_password,
                            }
                        )
                    ),
                )
                .json()
            )

            if api_call.get("message") != "User password changed":
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                logging.info("You successfully updated the user password.")
        else:
            logging.error("There is no id defined.")
            raise ValueError

    def switch_current_user_context(self, org_id: int):
        """The method includes a functionality to switch the current user context to the given organization

        Args:
            org_id (int): Specify the organization id

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if org_id != 0:
            api_call: dict = (
                Api(self.grafana_api_model)
                .call_the_api(
                    f"{APIEndpoints.USER.value}/using/{org_id}",
                    RequestsMethods.POST,
                    json.dumps(dict()),
                )
                .json()
            )

            if api_call.get("message") != "Active organization changed":
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                logging.info("You successfully updated active organization.")
        else:
            logging.error("There is no org_id defined.")
            raise ValueError

    def get_user_organizations(self) -> list:
        """The method includes a functionality to get the current user organizations

        Raises:
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (list): Returns a list of organizations
        """

        api_call: list = (
            Api(self.grafana_api_model)
            .call_the_api(
                f"{APIEndpoints.USER.value}/orgs",
            )
            .json()
        )

        if api_call == list() or api_call[0].get("orgId") is None:
            logging.error(f"Check the error: {api_call}.")
            raise Exception
        else:
            return api_call

    def get_user_teams(self) -> list:
        """The method includes a functionality to get the current user teams

        Raises:
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (list): Returns a list of teams
        """

        api_call: list = (
            Api(self.grafana_api_model)
            .call_the_api(
                f"{APIEndpoints.USER.value}/teams",
            )
            .json()
        )

        if api_call == list() or api_call[0].get("id") is None:
            logging.error(f"Check the error: {api_call}.")
            raise Exception
        else:
            return api_call

    def star_a_dashboard(self, dashboard_id: int):
        """The method includes a functionality to star a dashboard for the current user

        Args:
            dashboard_id (int): Specify the dashboard id

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if dashboard_id != 0:
            api_call: dict = (
                Api(self.grafana_api_model)
                .call_the_api(
                    f"{APIEndpoints.USER.value}/stars/dashboard/{dashboard_id}",
                    RequestsMethods.POST,
                    json.dumps(dict()),
                )
                .json()
            )

            if api_call.get("message") != "Dashboard starred!":
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                logging.info("You successfully starred the corresponding dashboard.")
        else:
            logging.error("There is no org_id defined.")
            raise ValueError

    def unstar_a_dashboard(self, dashboard_id: int):
        """The method includes a functionality to unstar a dashboard for the current user

        Args:
            dashboard_id (int): Specify the dashboard id

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if dashboard_id != 0:
            api_call: dict = (
                Api(self.grafana_api_model)
                .call_the_api(
                    f"{APIEndpoints.USER.value}/stars/dashboard/{dashboard_id}",
                    RequestsMethods.DELETE,
                )
                .json()
            )

            if api_call.get("message") != "Dashboard unstarred":
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                logging.info("You successfully unstarred the corresponding dashboard.")
        else:
            logging.error("There is no org_id defined.")
            raise ValueError

    def get_auth_tokens(self) -> list:
        """The method includes a functionality to get the auth tokens for the current user

        Raises:
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (list): Returns a list of auth tokens of the current user
        """

        api_call: list = (
            Api(self.grafana_api_model)
            .call_the_api(
                f"{APIEndpoints.USER.value}/auth-tokens",
            )
            .json()
        )

        if api_call == list() or api_call[0].get("id") is None:
            logging.error(f"Check the error: {api_call}.")
            raise Exception
        else:
            return api_call

    def revoke_auth_token(self, auth_token_id: int):
        """The method includes a functionality to revoke a specified auth token of the current user

        Args:
            auth_token_id (int): Specify the auth_token_id

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if auth_token_id != 0:
            api_call: dict = (
                Api(self.grafana_api_model)
                .call_the_api(
                    f"{APIEndpoints.USER.value}/revoke-auth-token",
                    RequestsMethods.POST,
                    json.dumps(dict({"authTokenId": auth_token_id})),
                )
                .json()
            )

            if api_call.get("message") != "User auth token revoked":
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                logging.info("You successfully revoked the corresponding token.")
        else:
            logging.error("There is no auth_token_id defined.")
            raise ValueError



from enum import Enum
from typing import NamedTuple

# The constant includes all necessary error messages that can occurs, if you establish a connection to the Grafana API.
ERROR_MESSAGES: list = ["invalid API key"]


class APIEndpoints(Enum):
    """The class includes all necessary API endpoint strings to connect the Grafana API"""

    api_prefix = "/api"

    SEARCH = f"{api_prefix}/search"
    DASHBOARDS = f"{api_prefix}/dashboards"
    FOLDERS = f"{api_prefix}/folders"
    LEGACY_ALERTS = f"{api_prefix}/alerts"
    ALERT_NOTIFICATIONS = f"{api_prefix}/alert-notifications"
    ALERTS_ALERTMANAGER = f"{api_prefix}/alertmanager"
    ALERTS_PROMETHEUS = f"{api_prefix}/prometheus"
    ALERTS_RULER = f"{api_prefix}/ruler"
    ALERTS_TESTING = f"{api_prefix}/v1"
    ALERTS_NGALERT = f"{api_prefix}/v1/ngalert"
    DATASOURCES = f"{api_prefix}/datasources"
    DATASOURCE_QUERY = f"{api_prefix}/tsdb/query"
    SHORT_URLS = f"{api_prefix}/short-urls"
    ORGANISATION = f"{api_prefix}/org"
    ORGANISATIONS = f"{api_prefix}/orgs"
    USER = f"{api_prefix}/user"
    USERS = f"{api_prefix}/users"
    SNAPSHOTS = f"{api_prefix}/snapshots"
    DASHBOARD_SNAPSHOTS = f"{api_prefix}/dashboard/snapshots"
    PLAYLISTS = f"{api_prefix}/playlists"
    TEAMS = f"{api_prefix}/teams"
    QUERY_HISTORY = f"{api_prefix}/query-history"
    REPORTING = f"{api_prefix}/reports/email"
    LICENSING = f"{api_prefix}/licensing"
    FRONTEND = f"{api_prefix}/frontend"
    LOGIN = f"{api_prefix}/login"


class RequestsMethods(Enum):
    """The class includes all necessary methods to establish an HTTP/ HTTPS connection to the Grafana API endpoints"""

    GET = "GET"
    PUT = "PUT"
    POST = "POST"
    PATCH = "PATCH"
    DELETE = "DELETE"


class APIModel(NamedTuple):
    """The class includes all necessary variables to establish a connection to the Grafana API endpoints

    Args:
        host (str): Specify the host of the Grafana system
        token (str): Specify the access token of the Grafana system
        username (str): Specify the username of the Grafana system
        password (str): Specify the password of the Grafana system
    """

    host: str
    token: str = None
    username: str = None
    password: str = None


class DatasourceQuery(NamedTuple):
    """The class includes all necessary variables to specify a query for the datasource search endpoint

    Args:
        datasource_id (int): Specify the id of the data source
        raw_sql (str): Specify the raw SQL string to search inside the Grafana system
        ref_id (str): Specify a reference id of the search command (default A)
        interval_ms (int): Specify the time interval in milliseconds of output format (default 1000)
        max_data_points (int): Specify maximum amount of data points that dashboard panel can render (default 100)
        output_format (str): Specify the output format of the query (default time_series)
    """

    datasource_id: int
    raw_sql: str
    ref_id: str = "A"
    interval_ms: int = 1000
    max_data_points: int = 100
    output_format: str = "time_series"


class DatasourceRuleQuery(NamedTuple):
    """The class includes all necessary variables to specify a query for the datasource rule search endpoint

    Args:
        datasource_uid (str): Specify the uid of the data source
        model (dict): Specify the model of the search command
        query_type (str): Specify the query time of the search command
        ref_id (str): Specify a reference id of the search command
        relative_time_range (dict): Specify the related time range of the search command
    """

    datasource_uid: str
    model: dict
    query_type: str
    ref_id: str
    relative_time_range: dict


class Alert(NamedTuple):
    """The class includes all necessary variables to generate an alert object that is necessary to communicate with the Grafana alert endpoint

    Args:
        starts_at (str): Specify the start date of the alert
        ends_at (str): Specify end date of the alert
        annotations (dict): Specify the annotations of the alert
        generator_url (str):  Specify the url of the generator endpoint
        labels (dict): Specify labels of the alert
    """

    starts_at: str
    ends_at: str
    annotations: dict
    generator_url: str
    labels: dict


class Silence(NamedTuple):
    """The class includes all necessary variables to generate a silence object that is necessary to communicate with the Grafana silence endpoint

    Args:
        starts_at (str): Specify the start date of the silence
        created_by (str): Specify the name of the silence creator
        ends_at (str): Specify end date of the silence
        comment (str):  Specify a custom comment for the silence
        id (str): Specify an id for the silence
        matchers (dict): Specify matchers for the silence
    """

    starts_at: str
    created_by: str
    ends_at: str
    comment: str
    id: str
    matchers: dict


class AlertmanagerConfig(NamedTuple):
    """The class includes all necessary variables to generate an Alertmanager config object that is necessary to communicate and set up the Grafana Alertmanager endpoint

    Args:
        global_config (dict): Specify the global config of the Alertmanager
        inhibit_rules (list): Specify the inhibit rules of the Alertmanager
        mute_time_intervals (list): Specify the mute time intervals of the Alertmanager
        receivers (list):  Specify the receivers of the Alertmanager
        route (dict): Specify the route of the Alertmanager
        templates (list): Specify an Alertmanager template
    """

    global_config: dict
    inhibit_rules: list
    mute_time_intervals: list
    receivers: list
    route: dict
    templates: list


class AlertmanagerReceivers(NamedTuple):
    """The class includes all necessary variables to generate an Alertmanager receivers object that is necessary to communicate and set up the Grafana Alertmanager receivers endpoint

    Args:
        name (str): Specify the name of the receiver
        email_configs (list): Specify the email configuration of the receiver's
        grafana_managed_receiver_configs (list): Specify the Grafana managed receiver configuration of the receiver's
        opsgenie_configs (list):  Specify the ops genie configuration of the receiver's
        pagerduty_configs (dict): Specify the pager duty configuration of the receiver's
        pushover_configs (list): Specify the push over configuration of the receiver's
        slack_configs (list): Specify the Slack configuration of the receiver's
        victorops_configs (list): Specify the victor ops configuration of the receiver's
        webhook_configs (list): Specify the webhook configuration of the receiver's
        wechat_configs (list): Specify the wechaty configuration of the receiver's
    """

    name: str
    email_configs: list
    grafana_managed_receiver_configs: list
    opsgenie_configs: list
    pagerduty_configs: list
    pushover_configs: list
    slack_configs: list
    sns_configs: list
    victorops_configs: list
    webhook_configs: list
    wechat_configs: list


class RulerRule(NamedTuple):
    """The class includes all necessary variables to generate a Ruler rule object that is necessary to communicate and set up a Grafana Ruler rule

    Args:
        alert (str): Specify the name of the rule
        annotations (dict): Specify the annotations of the rule
        expr (str): Specify the expression of the rule
        grafana_alert (dict):  Specify the Grafana alert of the rule
        labels (dict): Specify labels of the rule
        record (str): Specify recode value of the rule
        for_id (int): Specify the id of the rule if you update an existing rule (default 0)
    """

    alert: str
    annotations: dict
    expr: str
    grafana_alert: dict
    labels: dict
    record: str
    for_id: int = 0


class UserObject(NamedTuple):
    """The class includes all necessary variables to generate a User object that is necessary to update a Grafana User

    Args:
        email (str): Specify the name of the rule
        name (str): Specify the annotations of the rule
        login (str): Specify the expression of the rule
        theme (str):  Specify the Grafana alert of the rule
    """

    email: str
    name: str
    login: str
    theme: str


class PlaylistObject(NamedTuple):
    """The class includes all necessary variables to generate a playlist object

    Args:
        name (str): Specify the name of the playlist
        interval (str): Specify the interval of the playlist
        items (list): Specify a list of PlaylistItemObjects
    """

    name: str
    interval: str
    items: list


class PlaylistItemObject(NamedTuple):
    """The class includes all necessary variables to generate a playlist item object that is necessary to update a playlist

    Args:
        type (str): Specify the type of the playlist item
        value (str): Specify the value of the playlist item
        order (int): Specify the order of the playlist item
        title (str): Specify the title of the playlist item
    """

    type: str
    value: str
    order: int
    title: str


class TeamObject(NamedTuple):
    """The class includes all necessary variables to generate a team object that is necessary to create a team

    Args:
        name (str): Specify the name of the team
        email (str): Specify the email of the team
        org_id (int): Specify the org_id of the team
    """

    name: str
    email: str
    org_id: int


class QueryDatasourceObject(NamedTuple):
    """The class includes all necessary variables to generate a query datasource object that is necessary to create a query history object

    Args:
        type (str): Specify the type of the datasource query
        uid (str): Specify the uid of the datasource query
    """

    type: str
    uid: str


class QueryObject(NamedTuple):
    """The class includes all necessary variables to generate a query object that is necessary to create a query history

    Args:
        ref_id (str): Specify the ref_id of the query history
        key (str): Specify the key of the query history
        scenario_id (str): Specify the scenario_id of the query history
        datasource (QueryDatasourceObject): Specify the datasource of the type QueryDatasourceObject
    """

    ref_id: str
    key: str
    scenario_id: str
    datasource: QueryDatasourceObject



import json
import logging

from .model import APIModel, APIEndpoints, RequestsMethods
from .folder import Folder
from .api import Api


class Dashboard:
    """The class includes all necessary methods to access the Grafana dashboard API endpoints

    Args:
        grafana_api_model (APIModel): Inject a Grafana API model object that includes all necessary values and information

    Attributes:
        grafana_api_model (APIModel): This is where we store the grafana_api_model
    """

    def __init__(self, grafana_api_model: APIModel):
        self.grafana_api_model = grafana_api_model

    def create_or_update_dashboard(
        self,
        dashboard_path: str,
        dashboard_json: dict,
        message: str,
        overwrite: bool = False,
    ):
        """The method includes a functionality to create the specified dashboard

        Args:
            dashboard_path (str): Specify the dashboard path in which the dashboard is to be placed
            dashboard_json (dict): Specify the inserted dashboard as dict
            message (str): Specify the message that should be injected as commit message inside the dashboard
            overwrite (bool): Should the already existing dashboard be overwritten (default False)

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if len(dashboard_path) != 0 and dashboard_json != dict() and len(message) != 0:
            folder_id: int = Folder(
                self.grafana_api_model
            ).get_folder_id_by_dashboard_path(dashboard_path)

            dashboard_json_complete: dict = {
                "dashboard": dashboard_json,
                "folderId": folder_id,
                "message": message,
                "overwrite": overwrite,
            }

            api_call: dict = (
                Api(self.grafana_api_model)
                .call_the_api(
                    f"{APIEndpoints.DASHBOARDS.value}/db",
                    RequestsMethods.POST,
                    json.dumps(dashboard_json_complete),
                )
                .json()
            )

            if api_call.get("status") != "success":
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                logging.info("You successfully deployed the dashboard.")
        else:
            logging.error(
                "There is no dashboard_path or dashboard_json or message defined."
            )
            raise ValueError

    def delete_dashboard_by_name_and_path(
        self, dashboard_name: str, dashboard_path: str
    ):
        """The method includes a functionality to delete the specified dashboard inside the model

        Args:
            dashboard_name (str): Specify the dashboard name of the deleted dashboard
            dashboard_path (str): Specify the dashboard path in which the dashboard is to be placed

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if len(dashboard_name) != 0 and len(dashboard_path) != 0:
            dashboard_uid: dict = self.get_dashboard_uid_and_id_by_name_and_folder(
                dashboard_name, dashboard_path
            )

            if len(dashboard_uid) != 0:
                api_call: dict = (
                    Api(self.grafana_api_model)
                    .call_the_api(
                        f"{APIEndpoints.DASHBOARDS.value}/uid/{dashboard_uid.get('uid')}",
                        RequestsMethods.DELETE,
                    )
                    .json()
                )

                if f"Dashboard {dashboard_name} deleted" != api_call.get("message"):
                    logging.error(f"Please, check the error: {api_call}.")
                    raise Exception
                else:
                    logging.info("You successfully destroyed the dashboard.")
            else:
                logging.error("Nothing to delete. There is no dashboard available.")
                raise ValueError
        else:
            logging.error("There is no dashboard_name or dashboard_path defined.")
            raise ValueError

    def get_dashboard_by_uid(self, uid: str) -> dict:
        """The method includes a functionality to get the dashboard from the specified uid

        Args:
            uid (str): Specify the uid of the dashboard

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns the dashboard
        """

        if len(uid) != 0:
            api_call: dict = (
                Api(self.grafana_api_model)
                .call_the_api(f"{APIEndpoints.DASHBOARDS.value}/uid/{uid}")
                .json()
            )

            if api_call.get("dashboard") is None:
                logging.error(f"Please, check the error: {api_call}.")
                raise Exception
            else:
                return api_call
        else:
            logging.error("There is no dashboard uid defined.")
            raise ValueError

    def get_dashboard_home(self) -> dict:
        """The method includes a functionality to get the home dashboard

        Raises:
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns the home dashboard
        """

        api_call: dict = (
            Api(self.grafana_api_model)
            .call_the_api(f"{APIEndpoints.DASHBOARDS.value}/home")
            .json()
        )

        if api_call.get("dashboard") is None:
            logging.error(f"Please, check the error: {api_call}.")
            raise Exception
        else:
            return api_call

    def get_dashboard_tags(self) -> list:
        """The method includes a functionality to get the all tags of all dashboards

        Raises:
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (list): Returns all dashboard tags
        """

        api_call: list = (
            Api(self.grafana_api_model)
            .call_the_api(f"{APIEndpoints.DASHBOARDS.value}/tags")
            .json()
        )

        if api_call == list() or api_call[0].get("term") is None:
            logging.error(f"Please, check the error: {api_call}.")
            raise Exception
        else:
            return api_call

    def get_dashboard_uid_and_id_by_name_and_folder(
        self, dashboard_name: str, dashboard_path: str
    ) -> dict:
        """The method includes a functionality to extract the dashboard uid specified inside the model

        Args:
            dashboard_name (str): Specify the dashboard name of the dashboard
            dashboard_path (str): Specify the dashboard path of the dashboard

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns the dashboard uid and the id
        """

        if len(dashboard_name) != 0 and len(dashboard_path) != 0:
            folder_id: int = Folder(
                self.grafana_api_model
            ).get_folder_id_by_dashboard_path(dashboard_path)

            search_query: str = f"{APIEndpoints.SEARCH.value}?folderIds={folder_id}&query={dashboard_name}"
            dashboard_meta: list = (
                Api(self.grafana_api_model).call_the_api(search_query).json()
            )

            for dashboard_meta_object in dashboard_meta:
                if dashboard_meta_object.get("title") is not None:
                    if dashboard_meta_object.get("title") == dashboard_name:
                        if (
                            dashboard_meta_object.get("uid") is not None
                            and dashboard_meta_object.get("id") is not None
                        ):
                            return dict(
                                {
                                    "uid": dashboard_meta_object.get("uid"),
                                    "id": dashboard_meta_object.get("id"),
                                }
                            )
                        else:
                            logging.error("There is no uid or id defined.")
                            raise ValueError
                else:
                    logging.error("There is no title defined.")
                    raise ValueError
        else:
            logging.error("There is no dashboard_name or dashboard_path defined.")
            raise ValueError

    def get_dashboard_permissions(self, id: int) -> list:
        """The method includes a functionality to extract the dashboard permissions based on the specified id

        Args:
            id (int): Specify the id of the dashboard

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (list): Returns the dashboard permissions of a dashboard as list
        """

        if id != 0:
            api_call: list = (
                Api(self.grafana_api_model)
                .call_the_api(f"{APIEndpoints.DASHBOARDS.value}/id/{id}/permissions")
                .json()
            )

            if api_call == list() or api_call[0].get("role") is None:
                logging.error(f"Please, check the error: {api_call}.")
                raise Exception
            else:
                return api_call
        else:
            logging.error("There is no dashboard uid defined.")
            raise ValueError

    def update_dashboard_permissions(self, id: int, permission_json: dict):
        """The method includes a functionality to update the dashboard permissions based on the specified id and the permission json document

        Args:
            id (int): Specify the id of the dashboard
            permission_json (dict): Specify the inserted permissions as dict

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if id != 0 and len(permission_json) != 0:
            api_call: dict = (
                Api(self.grafana_api_model)
                .call_the_api(
                    f"{APIEndpoints.DASHBOARDS.value}/id/{id}/permissions",
                    RequestsMethods.POST,
                    json.dumps(permission_json),
                )
                .json()
            )

            if api_call.get("message") != "Dashboard permissions updated":
                logging.error(f"Please, check the error: {api_call}.")
                raise Exception
            else:
                logging.info("You successfully modified the dashboard permissions.")
        else:
            logging.error("There is no dashboard uid or permission json defined.")
            raise ValueError

    def get_dashboard_versions(self, id: int) -> list:
        """The method includes a functionality to extract the versions of a dashboard based on the specified id

        Args:
            id (int): Specify the id of the dashboard

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (list): Returns all dashboard versions of a dashboard as list
        """

        if id != 0:
            api_call: list = (
                Api(self.grafana_api_model)
                .call_the_api(
                    f"{APIEndpoints.DASHBOARDS.value}/id/{id}/versions",
                )
                .json()
            )

            if api_call == list() or api_call[0].get("id") is None:
                logging.error(f"Please, check the error: {api_call}.")
                raise Exception
            else:
                return api_call
        else:
            logging.error("There is no dashboard uid defined.")
            raise ValueError

    def get_dashboard_version(self, id: int, version_id: int) -> dict:
        """The method includes a functionality to extract a specified version of a dashboard based on the specified dashboard id and a version_id of the dashboard

        Args:
            id (int): Specify the id of the dashboard
            version_id (int): Specify the version_id of a dashboard

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (list): Returns a dashboard version of a dashboard as dict
        """

        if id != 0 and version_id != 0:
            api_call: dict = (
                Api(self.grafana_api_model)
                .call_the_api(
                    f"{APIEndpoints.DASHBOARDS.value}/id/{id}/versions/{version_id}",
                )
                .json()
            )

            if api_call == dict() or api_call.get("id") is None:
                logging.error(f"Please, check the error: {api_call}.")
                raise Exception
            else:
                return api_call
        else:
            logging.error("There is no dashboard uid or version_id defined.")
            raise ValueError

    def restore_dashboard_version(self, id: int, version: dict):
        """The method includes a functionality to restore a specified version of a dashboard based on the specified dashboard uid and a version as dict of the dashboard

        Args:
            id (int): Specify the id of the dashboard
            version (dict): Specify the version_id of a dashboard

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if id != 0 and version != dict():
            api_call: dict = (
                Api(self.grafana_api_model)
                .call_the_api(
                    f"{APIEndpoints.DASHBOARDS.value}/id/{id}/restore",
                    RequestsMethods.POST,
                    json.dumps(version),
                )
                .json()
            )

            if (
                api_call.get("status") != "success"
                or api_call.get("message") is not None
            ):
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                logging.info("You successfully restored the dashboard.")
        else:
            logging.error("There is no dashboard uid or version_id defined.")
            raise ValueError

    def calculate_dashboard_diff(
        self,
        dashboard_id_and_version_base: dict,
        dashboard_id_and_version_new: dict,
        diff_type: str = "json",
    ) -> str:
        """The method includes a functionality to calculate the diff of specified versions of a dashboard based on the specified dashboard uid and the selected version of the base dashboard and the new dashboard and the diff type (basic or json)

        Args:
            dashboard_id_and_version_base (dict): Specify the version and id of the base dashboard
            dashboard_id_and_version_new (dict): Specify the version and id of the new dashboard
            diff_type (str): Specify the diff type (basic or json) (default json)

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (str): Returns the difference of the two specified dashboards
        """
        possible_diff_types: list = list(["basic", "json"])

        if diff_type.lower() in possible_diff_types:
            if (
                dashboard_id_and_version_base != dict()
                and dashboard_id_and_version_new != 0
            ):
                diff_object: dict = dict()
                diff_object.update(dashboard_id_and_version_base)
                diff_object.update(dashboard_id_and_version_new)
                diff_object.update({"diffType": diff_type.lower()})

                api_call: any = Api(self.grafana_api_model).call_the_api(
                    f"{APIEndpoints.DASHBOARDS.value}/calculate-diff",
                    RequestsMethods.POST,
                    json.dumps(diff_object),
                )

                if api_call.status_code != 200:
                    logging.error(f"Check the error: {api_call.text}.")
                    raise Exception
                else:
                    return api_call.text
            else:
                logging.error(
                    "There is no dashboard_uid_and_version_base or dashboard_uid_and_version_new defined."
                )
                raise ValueError
        else:
            logging.error(
                f"The diff_type: {diff_type.lower()} is not valid. Please specify a valid value."
            )
            raise ValueError



import logging

import requests

from .model import APIModel, APIEndpoints
from .api import Api


class OtherHTTP:
    """The class includes all necessary methods to access other Grafana API endpoints

    Args:
        grafana_api_model (APIModel): Inject a Grafana API model object that includes all necessary values and information

    Attributes:
        grafana_api_model (APIModel): This is where we store the grafana_api_model
    """

    def __init__(self, grafana_api_model: APIModel):
        self.grafana_api_model = grafana_api_model

    def get_frontend_settings(self) -> dict:
        """The method includes a functionality to get the frontend settings

        Raises:
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns the corresponding frontend settings
        """

        api_call: dict = (
            Api(self.grafana_api_model)
            .call_the_api(f"{APIEndpoints.FRONTEND.value}/settings")
            .json()
        )

        if api_call == dict():
            logging.error(f"Check the error: {api_call}.")
            raise Exception
        else:
            return api_call

    def renew_login_session_based_on_remember_cookie(self):
        """The method includes a functionality to renew the login session based on the remember cookie

        Raises:
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        api_call: str = (
            Api(self.grafana_api_model)
            .call_the_api(f"{APIEndpoints.LOGIN.value}/ping")
            .text
        )

        if api_call != "Logged in":
            logging.error(f"Check the error: {api_call}.")
            raise Exception
        else:
            logging.info("You successfully destroyed the dashboard snapshot.")

    def get_health_status(self) -> dict:
        """The method includes a functionality to get the health information

        Raises:
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns the health information
        """

        api_call: dict = requests.get(
            f"{self.grafana_api_model.host}/api/health"
        ).json()

        if api_call == dict() or api_call.get("commit") is None:
            logging.error(f"Check the error: {api_call}.")
            raise Exception
        else:
            return api_call

    def get_metrics(self) -> str:
        """The method includes a functionality to get the Grafana metrics information

        Raises:
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (str): Returns the metrics information
        """

        api_call: str = requests.get(f"{self.grafana_api_model.host}/metrics").text

        if len(api_call) == 0 or api_call is None:
            logging.error(f"Check the error: {api_call}.")
            raise Exception
        else:
            return api_call



import logging
import json

import requests

from .model import RequestsMethods, ERROR_MESSAGES, APIModel


class Api:
    """The class includes all necessary methods to make API calls to the Grafana API endpoints

    Args:
        grafana_api_model (APIModel): Inject a Grafana API model object that includes all necessary values and information

    Attributes:
        grafana_api_model (APIModel): This is where we store the grafana_api_model
    """

    def __init__(self, grafana_api_model: APIModel):
        self.grafana_api_model = grafana_api_model

    def call_the_api(
        self,
        api_call: str,
        method: RequestsMethods = RequestsMethods.GET,
        json_complete: str = None,
        timeout: float = None,
    ) -> any:
        """The method execute a defined API call against the Grafana endpoints

        Args:
            api_call (str): Specify the API call endpoint
            method (RequestsMethods): Specify the used method (default GET)
            json_complete (str): Specify the inserted JSON as string
            timeout (float): Specify the timeout for the corresponding API call

        Raises:
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (any): Returns the value of the api call
        """

        api_url: str = f"{self.grafana_api_model.host}{api_call}"
        headers: dict = dict(
            {"Authorization": f"Bearer {self.grafana_api_model.token}"}
        )

        if (
            self.grafana_api_model.username is not None
            and self.grafana_api_model.password is not None
        ):
            url: str = (
                f"{self.grafana_api_model.username}:{self.grafana_api_model.password}@"
            )
            api_url = api_url.replace("https://", f"https://{url}")
            api_url = api_url.replace("http://", f"http://{url}")
        else:
            headers["Content-Type"] = "application/json"

        try:
            if method.value == RequestsMethods.GET.value:
                return Api.__check_the_api_call_response(
                    requests.get(api_url, headers=headers, timeout=timeout)
                )
            elif method.value == RequestsMethods.PUT.value:
                if json_complete is not None:
                    return Api.__check_the_api_call_response(
                        requests.put(
                            api_url,
                            data=json_complete,
                            headers=headers,
                            timeout=timeout,
                        )
                    )
                else:
                    logging.error("Please define the json_complete.")
                    raise Exception
            elif method.value == RequestsMethods.POST.value:
                if json_complete is not None:
                    return Api.__check_the_api_call_response(
                        requests.post(
                            api_url,
                            data=json_complete,
                            headers=headers,
                            timeout=timeout,
                        )
                    )
                else:
                    logging.error("Please define the json_complete.")
                    raise Exception
            elif method.value == RequestsMethods.PATCH.value:
                if json_complete is not None:
                    return Api.__check_the_api_call_response(
                        requests.patch(
                            api_url,
                            data=json_complete,
                            headers=headers,
                            timeout=timeout,
                        )
                    )
                else:
                    logging.error("Please define the json_complete.")
                    raise Exception
            elif method.value == RequestsMethods.DELETE.value:
                return Api.__check_the_api_call_response(
                    requests.delete(api_url, headers=headers, timeout=timeout)
                )
            else:
                logging.error("Please define a valid method.")
                raise Exception
        except Exception as e:
            raise e

    @staticmethod
    def __check_the_api_call_response(response: any = None) -> any:
        """The method includes a functionality to check the output of API call method for errors

        Args:
            response (any): Specify the inserted response

        Raises:
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (any): Returns the value of the api call
        """

        if Api.__check_if_valid_json(response.text):
            if len(response.text) != 0 and type(response.json()) == dict:
                if (
                    "message" in response.json().keys()
                    and response.json()["message"] in ERROR_MESSAGES
                ):
                    logging.error(response.json()["message"])
                    raise requests.exceptions.ConnectionError

        return response

    @staticmethod
    def __check_if_valid_json(response: any) -> bool:
        """The method includes a functionality to check if the response json is valid

        Args:
            response (any): Specify the inserted response json

        Returns:
            api_call (bool): Returns if the json is valid or not
        """

        try:
            json.loads(response)
        except ValueError:
            return False
        return True



import datetime
import json
import logging

from .model import (
    APIModel,
    APIEndpoints,
    RequestsMethods,
    Silence,
    AlertmanagerConfig,
)
from .api import Api


class Alerting:
    """The class includes all necessary methods to access the Grafana alerting API endpoints

    Args:
        grafana_api_model (APIModel): Inject a Grafana API model object that includes all necessary values and information

    Attributes:
        grafana_api_model (APIModel): This is where we store the grafana_api_model
    """

    def __init__(self, grafana_api_model: APIModel):
        self.grafana_api_model = grafana_api_model

    def get_alertmanager_alerts(self, recipient: any = "grafana") -> list:
        """The method includes a functionality to get the Alertmanager alerts specified by the recipient

        Args:
            recipient (any): Specify the recipient datasource id of the alerts (default grafana)

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (list): Returns the list of Alertmanager alerts
        """

        if (type(recipient) == int and recipient != 0) or (
            type(recipient) == str and len(recipient) != 0
        ):
            api_call: list = (
                Api(self.grafana_api_model)
                .call_the_api(
                    f"{APIEndpoints.ALERTS_ALERTMANAGER.value}/{recipient}/api/v2/alerts",
                )
                .json()
            )

            if api_call == list() or api_call[0].get("receivers") is None:
                logging.error(f"Please, check the error: {api_call}.")
                raise Exception
            else:
                return api_call
        else:
            logging.error("There is no recipient defined.")
            raise ValueError

    def create_or_update_alertmanager_alerts(
        self, alerts: list, recipient: any = "grafana"
    ):
        """The method includes a functionality to create or update the Alertmanager alerts specified by the recipient and the alerts list

        Args:
            alerts (list): Specify a list of the alert objects
            recipient (any): Specify the recipient datasource id of the alerts (default grafana)

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if (
            (type(recipient) == int and recipient != 0)
            or (type(recipient) == str and len(recipient) != 0)
        ) and alerts != list():
            alerts_json_list: list = list()

            for alert in alerts:
                alert_json_dict: dict = dict(
                    {
                        "startsAt": alert.starts_at,
                        "endsAt": alert.ends_at,
                        "generatorURL": alert.generator_url,
                        "labels": alert.labels,
                        "annotations": alert.annotations,
                    }
                )
                alerts_json_list.append(alert_json_dict)

            api_call: dict = (
                Api(self.grafana_api_model)
                .call_the_api(
                    f"{APIEndpoints.ALERTS_ALERTMANAGER.value}/{recipient}/api/v2/alerts/groups",
                    RequestsMethods.POST,
                    json.dumps(alerts_json_list),
                )
                .json()
            )

            if api_call != dict():
                logging.error(f"Please, check the error: {api_call.get('message')}.")
                raise Exception
            else:
                logging.info("You successfully created alerts.")
        else:
            logging.error("There is no recipient or alerts defined.")
            raise ValueError

    def get_alertmanager_group_alerts(self, recipient: any = "grafana") -> list:
        """The method includes a functionality to get the Alertmanager group alerts specified by the recipient

        Args:
            recipient (any): Specify the recipient datasource id of the alerts (default grafana)

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (list): Returns the list of Alertmanager group alerts
        """

        if (type(recipient) == int and recipient != 0) or (
            type(recipient) == str and len(recipient) != 0
        ):
            api_call: list = (
                Api(self.grafana_api_model)
                .call_the_api(
                    f"{APIEndpoints.ALERTS_ALERTMANAGER.value}/{recipient}/api/v2/alerts",
                )
                .json()
            )

            if api_call == list() or api_call[0].get("alerts") is None:
                logging.error(f"Please, check the error: {api_call}.")
                raise Exception
            else:
                return api_call
        else:
            logging.error("There is no recipient defined.")
            raise ValueError

    def delete_alertmanager_silence_by_id(
        self, silence_id: str, recipient: any = "grafana"
    ):
        """The method includes a functionality to delete the Alertmanager silence specified by the silence id and the recipient

        Args:
            silence_id (str): Specify the silence id of the alerts
            recipient (any): Specify the recipient datasource id of the alerts (default grafana)

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if (
            (type(recipient) == int and recipient != 0)
            or (type(recipient) == str and len(recipient) != 0)
        ) and len(silence_id) != 0:
            api_call: dict = (
                Api(self.grafana_api_model)
                .call_the_api(
                    f"{APIEndpoints.ALERTS_ALERTMANAGER.value}/{recipient}/api/v2/silence/{silence_id}",
                    RequestsMethods.DELETE,
                )
                .json()
            )

            if api_call != dict():
                logging.error(f"Please, check the error: {api_call}.")
                raise Exception
            else:
                logging.info("You successfully deleted a silence.")
        else:
            logging.error("There is no recipient or silence_id defined.")
            raise ValueError

    def get_alertmanager_silence_by_id(
        self, silence_id: str, recipient: any = "grafana"
    ) -> dict:
        """The method includes a functionality to get the Alertmanager silence specified by the silence id and the recipient

        Args:
            silence_id (str): Specify the silence id of the alerts
            recipient (any): Specify the recipient datasource id of the alerts (default grafana)

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns the dict of Alertmanager silence alert
        """

        if (
            (type(recipient) == int and recipient != 0)
            or (type(recipient) == str and len(recipient) != 0)
        ) and len(silence_id) != 0:
            api_call: dict = (
                Api(self.grafana_api_model)
                .call_the_api(
                    f"{APIEndpoints.ALERTS_ALERTMANAGER.value}/{recipient}/api/v2/silence/{silence_id}",
                )
                .json()
            )

            if api_call == dict() or api_call.get("id") is None:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                return api_call
        else:
            logging.error("There is no recipient or silence_id defined.")
            raise ValueError

    def get_alertmanager_silences(self, recipient: any = "grafana") -> list:
        """The method includes a functionality to get all Alertmanager silences specified by the recipient

        Args:
            recipient (any): Specify the recipient datasource id of the alerts (default grafana)

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (list): Returns the list of Alertmanager silence alerts
        """

        if (type(recipient) == int and recipient != 0) or (
            type(recipient) == str and len(recipient) != 0
        ):
            api_call: list = (
                Api(self.grafana_api_model)
                .call_the_api(
                    f"{APIEndpoints.ALERTS_ALERTMANAGER.value}/{recipient}/api/v2/silences",
                )
                .json()
            )

            if api_call == list() or api_call[0].get("id") is None:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                return api_call
        else:
            logging.error("There is no recipient defined.")
            raise ValueError

    def create_or_update_alertmanager_silence(
        self, silence: Silence, recipient: any = "grafana"
    ) -> dict:
        """The method includes a functionality to create or update the Alertmanager silence specified by the silence object and the recipient

        Args:
            silence -> Specify the silence object
            recipient (any): Specify the recipient datasource id of the alerts (default grafana)

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns the dict of newly created silence alert
        """

        if (
            (type(recipient) == int and recipient != 0)
            or (type(recipient) == str and len(recipient) != 0)
        ) or silence is not None:
            silence_json_dict: dict = dict(
                {
                    "comment": silence.comment,
                    "createdBy": silence.created_by,
                    "endsAt": silence.ends_at,
                    "id": silence.id,
                    "matchers": silence.matchers,
                    "startsAt": silence.starts_at,
                }
            )

            api_call: dict = (
                Api(self.grafana_api_model)
                .call_the_api(
                    f"{APIEndpoints.ALERTS_ALERTMANAGER.value}/{recipient}/api/v2/silences",
                    RequestsMethods.POST,
                    json.dumps(silence_json_dict),
                )
                .json()
            )

            if api_call == dict() or api_call.get("id") is None:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                return api_call
        else:
            logging.error("There is no recipient or silence defined.")
            raise ValueError

    def get_alertmanager_status(self, recipient: str = "grafana") -> dict:
        """The method includes a functionality to get the Alertmanager status specified by the recipient

        Args:
            recipient (str): Specify the recipient datasource id of the alerts (default grafana)

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns the dict of the Alertmanager status
        """

        if (type(recipient) == int and recipient != 0) or (
            type(recipient) == str and len(recipient) != 0
        ):
            api_call: dict = (
                Api(self.grafana_api_model)
                .call_the_api(
                    f"{APIEndpoints.ALERTS_ALERTMANAGER.value}/{recipient}/api/v2/status",
                )
                .json()
            )

            if api_call == dict() or api_call.get("config") is None:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                return api_call
        else:
            logging.error("There is no recipient defined.")
            raise ValueError

    def delete_alertmanager_config(self, recipient: any = "grafana"):
        """The method includes a functionality to delete the Alertmanager config specified by the recipient

        Args:
            recipient (any): Specify the recipient datasource id of the alerts (default grafana)

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if (type(recipient) == int and recipient != 0) or (
            type(recipient) == str and len(recipient) != 0
        ):
            api_call: dict = (
                Api(self.grafana_api_model)
                .call_the_api(
                    f"{APIEndpoints.ALERTS_ALERTMANAGER.value}/{recipient}/config/api/v1/alerts",
                    RequestsMethods.DELETE,
                )
                .json()
            )

            if (
                api_call.get("message")
                != "configuration deleted; the default is applied"
            ):
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                logging.info("You successfully deleted a alerting config.")
        else:
            logging.error("There is no recipient defined.")
            raise ValueError

    def get_alertmanager_config(self, recipient: any = "grafana") -> dict:
        """The method includes a functionality to get the Alertmanager config specified by the recipient

        Args:
            recipient (any): Specify the recipient datasource id of the alerts (default grafana)

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns the dict of the Alertmanager config
        """

        if (type(recipient) == int and recipient != 0) or (
            type(recipient) == str and len(recipient) != 0
        ):
            api_call: dict = (
                Api(self.grafana_api_model)
                .call_the_api(
                    f"{APIEndpoints.ALERTS_ALERTMANAGER.value}/{recipient}/config/api/v1/alerts",
                )
                .json()
            )

            if api_call == dict() or api_call.get("alertmanager_config") is None:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                return api_call
        else:
            logging.error("There is no recipient defined.")
            raise ValueError

    def create_or_update_alertmanager_config(
        self,
        alertmanager_config: AlertmanagerConfig,
        recipient: any = "grafana",
        template_files: dict = None,
    ):
        """The method includes a functionality to create or update the Alertmanager config specified by the Alertmanager config object, recipient and template_files

        Args:
            alertmanager_config (AlertmanagerConfig): Specify the Alertmanager config object
            recipient (any): Specify the recipient datasource id of the alerts (default grafana)
            template_files(dict): Specify the optional template files (default None)

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if (
            (type(recipient) == int and recipient != 0)
            or (type(recipient) == str and len(recipient) != 0)
        ) and alertmanager_config is not None:
            alertmanager_configuration_json_dict: dict = dict()

            alertmanager_configuration_json_dict["alertmanager_config"] = dict(
                {
                    "global": alertmanager_config.global_config,
                    "inhibit_rules": alertmanager_config.inhibit_rules,
                    "mute_time_intervals": alertmanager_config.mute_time_intervals,
                    "receivers": alertmanager_config.receivers,
                    "route": alertmanager_config.route,
                    "templates": alertmanager_config.templates,
                }
            )

            if template_files is not None:
                alertmanager_configuration_json_dict["template_files"] = template_files

            api_call: dict = (
                Api(self.grafana_api_model)
                .call_the_api(
                    f"{APIEndpoints.ALERTS_ALERTMANAGER.value}/{recipient}/config/api/v1/alerts",
                    RequestsMethods.POST,
                    json.dumps(alertmanager_configuration_json_dict),
                )
                .json()
            )

            if api_call == dict() or api_call.get("message") != "configuration created":
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                logging.info("You successfully created an Alertmanager alert config.")
        else:
            logging.error("There is no recipient or alertmanager_config defined.")
            raise ValueError

    def test_alertmanager_receivers(
        self, alert: dict, receivers: list, recipient: any = "grafana"
    ):
        """The method includes a functionality to test the Alertmanager receivers specified by the alert dict, receivers object and the recipient

        Args:
            alert (dict): Specify the alert dict
            receivers (list): Specify the list of AlertmanagerReceivers objects
            recipient (any): Specify the recipient datasource id of the alerts (default grafana)

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if (
            (
                (type(recipient) == int and recipient != 0)
                or (type(recipient) == str and len(recipient) != 0)
            )
            and alert != dict()
            and receivers is not None
        ):
            alertmanager_receivers_json_dict: dict = dict()
            receivers_list: list = list()

            alertmanager_receivers_json_dict["alert"] = alert

            for receiver in receivers:
                receivers_list.append(
                    dict(
                        {
                            "name": receiver.name,
                            "email_configs": receiver.email_configs,
                            "grafana_managed_receiver_configs": receiver.grafana_managed_receiver_configs,
                            "opsgenie_configs": receiver.opsgenie_configs,
                            "pagerduty_configs": receiver.pagerduty_configs,
                            "pushover_configs": receiver.pushover_configs,
                            "slack_configs": receiver.slack_configs,
                            "sns_configs": receiver.sns_configs,
                            "victorops_configs": receiver.victorops_configs,
                            "webhook_configs": receiver.webhook_configs,
                            "wechat_configs": receiver.wechat_configs,
                        }
                    )
                )

            alertmanager_receivers_json_dict["receivers"] = receivers_list

            api_call: any = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.ALERTS_ALERTMANAGER.value}/{recipient}/config/api/v1/receivers/test",
                RequestsMethods.POST,
                json.dumps(alertmanager_receivers_json_dict),
            )

            status_code: int = api_call.status_code
            api_call_dict: dict = api_call.json()

            alert_manager_status_dict: dict = dict(
                {
                    200: "You successfully tested Grafana managed receivers.",
                    207: "You successfully tested Grafana managed receivers (Multi Status).",
                    400: f"Validation error. Check the error: {api_call_dict.get('message')}.",
                    403: "Permission denied. Please, check the access rights.",
                    404: "Alert manager not found. Please, check the alert manager configuration.",
                    408: f"Failure. Check the error: {api_call_dict.get('message')}.",
                    409: "Alert manager not ready. Please, check the alert manager configuration.",
                }
            )

            if 200 <= status_code <= 207:
                logging.info(alert_manager_status_dict.get(status_code))
            elif 400 <= status_code <= 409:
                logging.error(alert_manager_status_dict.get(status_code))
                raise Exception
            else:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
        else:
            logging.error("There is no recipient, alert or receivers defined.")
            raise ValueError

    def get_prometheus_alerts(self, recipient: any = "grafana") -> dict:
        """The method includes a functionality to get all prometheus alerts specified by the recipient

        Args:
            recipient (any): Specify the recipient datasource id of the alerts (default grafana)

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns the dict of the prometheus alerts
        """

        if (type(recipient) == int and recipient != 0) or (
            type(recipient) == str and len(recipient) != 0
        ):
            api_call: dict = (
                Api(self.grafana_api_model)
                .call_the_api(
                    f"{APIEndpoints.ALERTS_PROMETHEUS.value}/{recipient}/api/v1/alerts",
                )
                .json()
            )

            if api_call == dict() or api_call.get("data") is None:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                return api_call
        else:
            logging.error("There is no recipient defined.")
            raise ValueError

    def get_prometheus_rules(self, recipient: any = "grafana") -> dict:
        """The method includes a functionality to get all prometheus rules specified by the recipient

        Args:
            recipient (any): Specify the recipient datasource id of the alerts (default grafana)

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns the dict of the prometheus rules
        """

        if (type(recipient) == int and recipient != 0) or (
            type(recipient) == str and len(recipient) != 0
        ):
            api_call: dict = (
                Api(self.grafana_api_model)
                .call_the_api(
                    f"{APIEndpoints.ALERTS_PROMETHEUS.value}/{recipient}/api/v1/rules",
                )
                .json()
            )

            if api_call == dict() or api_call.get("data") is None:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                return api_call
        else:
            logging.error("There is no recipient defined.")
            raise ValueError

    def get_ruler_rules(self, recipient: str = "grafana") -> dict:
        """The method includes a functionality to get all ruler rules specified by the recipient

        Args:
            recipient (str): Specify the recipient datasource id of the alerts (default grafana)

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns the dict of the ruler rules
        """

        if (type(recipient) == int and recipient != 0) or (
            type(recipient) == str and len(recipient) != 0
        ):
            api_call: dict = (
                Api(self.grafana_api_model)
                .call_the_api(
                    f"{APIEndpoints.ALERTS_RULER.value}/{recipient}/api/v1/rules",
                )
                .json()
            )

            if api_call == dict():
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                return api_call
        else:
            logging.error("There is no recipient defined.")
            raise ValueError

    def delete_ruler_namespace(self, namespace: str, recipient: any = "grafana"):
        """The method includes a functionality to delete a ruler namespace specified by the namespace name and the recipient

        Args:
            namespace (str): Specify the namespace name
            recipient (any): Specify the recipient datasource id of the alerts (default grafana)

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if (
            (type(recipient) == int and recipient != 0)
            or (type(recipient) == str and len(recipient) != 0)
        ) and len(namespace) != 0:
            api_call: dict = (
                Api(self.grafana_api_model)
                .call_the_api(
                    f"{APIEndpoints.ALERTS_RULER.value}/{recipient}/api/v1/rules/{namespace}",
                    RequestsMethods.DELETE,
                )
                .json()
            )

            if api_call != dict():
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                logging.info("You successfully deleted a ruler namespace.")
        else:
            logging.error("There is no recipient or namespace defined.")
            raise ValueError

    def get_ruler_groups_by_namespace(
        self, namespace: str, recipient: any = "grafana"
    ) -> dict:
        """The method includes a functionality to get all ruler groups specified by the namespace name and the recipient

        Args:
            namespace (str): Specify the namespace name
            recipient (any): Specify the recipient datasource id of the alerts (default grafana)

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns the dict of the ruler groups
        """

        if (
            (type(recipient) == int and recipient != 0)
            or (type(recipient) == str and len(recipient) != 0)
        ) and len(namespace) != 0:
            api_call: dict = (
                Api(self.grafana_api_model)
                .call_the_api(
                    f"{APIEndpoints.ALERTS_RULER.value}/{recipient}/api/v1/rules/{namespace}",
                )
                .json()
            )

            if api_call == dict():
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                return api_call
        else:
            logging.error("There is no recipient or namespace defined.")
            raise ValueError

    def create_or_update_ruler_group_by_namespace(
        self,
        namespace: str,
        group_name: str,
        rules: list,
        recipient: any = "grafana",
        interval: int = 0,
    ):
        """The method includes a functionality to create or update a ruler group specified by the namespace name, a ruler group name, a ruler rule object list, the recipient and an interval

        Args:
            namespace (str): Specify the namespace name
            group_name (str): Specify the ruler group name
            rules (list): Specify the ruler rule object list
            recipient (any): Specify the recipient datasource id of the alerts (default grafana)
            interval (int): Specify the interval of the ruler (default 0)

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if (
            (
                (type(recipient) == int and recipient != 0)
                or (type(recipient) == str and len(recipient) != 0)
            )
            and len(namespace) != 0
            and len(group_name) != 0
            and rules != list()
        ):
            rules_json_list: list = list()

            for rule in rules:
                rule_json_dict: dict = dict(
                    {
                        "alert": rule.alert,
                        "annotations": rule.annotations,
                        "expr": rule.expr,
                        "for": rule.for_id,
                        "grafana_alert": rule.grafana_alert,
                        "labels": rule.labels,
                        "record": rule.record,
                    }
                )
                rules_json_list.append(rule_json_dict)

            api_call: dict = (
                Api(self.grafana_api_model)
                .call_the_api(
                    f"{APIEndpoints.ALERTS_RULER.value}/{recipient}/api/v1/rules/{namespace}",
                    RequestsMethods.POST,
                    json.dumps(
                        {
                            "interval": interval,
                            "name": group_name,
                            "rules": rules_json_list,
                        }
                    ),
                )
                .json()
            )

            if api_call != dict():
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                logging.info("You successfully created an ruler group.")
        else:
            logging.error("There is no recipient, namespace, name or rules defined.")
            raise ValueError

    def delete_ruler_group(
        self, namespace: str, group_name: str, recipient: any = "grafana"
    ):
        """The method includes a functionality to delete a ruler group specified by the namespace name, a ruler group name and the recipient

        Args:
            namespace (str): Specify the namespace name
            group_name (str): Specify the ruler group name
            recipient (any): Specify the recipient datasource id of the alerts (default grafana)

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if (
            (
                (type(recipient) == int and recipient != 0)
                or (type(recipient) == str and len(recipient) != 0)
            )
            and len(namespace) != 0
            and len(group_name) != 0
        ):
            api_call: dict = (
                Api(self.grafana_api_model)
                .call_the_api(
                    f"{APIEndpoints.ALERTS_RULER.value}/{recipient}/api/v1/rules/{namespace}/{group_name}",
                    RequestsMethods.DELETE,
                )
                .json()
            )

            if api_call != dict():
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                logging.info("You successfully deleted a ruler group.")
        else:
            logging.error("There is no recipient, namespace or group_name defined.")
            raise ValueError

    def get_ruler_group(
        self, namespace: str, group_name: str, recipient: any = "grafana"
    ) -> dict:
        """The method includes a functionality to get a ruler group specified by the namespace name, a ruler group name and the recipient

        Args:
            namespace (str): Specify the namespace name
            group_name (str): Specify the ruler group name
            recipient (any): Specify the recipient datasource id of the alerts (default grafana)

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns the dict of all ruler groups
        """

        if (
            (
                (type(recipient) == int and recipient != 0)
                or (type(recipient) == str and len(recipient) != 0)
            )
            and len(namespace) != 0
            and len(group_name) != 0
        ):
            api_call: dict = (
                Api(self.grafana_api_model)
                .call_the_api(
                    f"{APIEndpoints.ALERTS_RULER.value}/{recipient}/api/v1/rules/{namespace}/{group_name}",
                )
                .json()
            )

            if api_call == dict():
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                return api_call
        else:
            logging.error("There is no recipient, namespace or group_name defined.")
            raise ValueError

    def test_rule(self, data_query: list) -> dict:
        """The method includes a functionality to test a rule specified by a list of datasource rule query objects

        Args:
            data_query (list): Specify a list of datasource rule query objects

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (str): Returns the result of the specified query
        """

        if data_query != list():
            datasource_rule_query_objects_json: list = list()
            datasource_rule_query_object_json: dict = dict()

            for datasource_rule_query_object in data_query:
                datasource_rule_query_object_json[
                    "datasourceUid"
                ] = datasource_rule_query_object.datasource_uid
                datasource_rule_query_object_json[
                    "model"
                ] = datasource_rule_query_object.model
                datasource_rule_query_object_json[
                    "queryType"
                ] = datasource_rule_query_object.query_type
                datasource_rule_query_object_json[
                    "refId"
                ] = datasource_rule_query_object.ref_id
                datasource_rule_query_object_json[
                    "relativeTimeRange"
                ] = datasource_rule_query_object.relative_time_range
                datasource_rule_query_objects_json.append(
                    datasource_rule_query_object_json
                )

            api_call: dict = (
                Api(self.grafana_api_model)
                .call_the_api(
                    "/api/v1/eval",
                    RequestsMethods.POST,
                    json.dumps(
                        {
                            "data": datasource_rule_query_objects_json,
                            "now": str(datetime.datetime.now()),
                        }
                    ),
                )
                .json()
            )

            if api_call == dict() or api_call.get("message") is not None:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                return api_call
        else:
            logging.error("There is no data_query defined.")
            raise ValueError

    def test_recipient_rule(
        self, expr: str, condition: str, data_query: list, recipient: any = "grafana"
    ) -> dict:
        """The method includes a functionality to test a recipient role specified by the expr, the condition, a list of data queries and the recipient

        Args:
            expr (str): Specify a list of datasource rule query objects
            condition (str): Specify the condition
            data_query (list): Specify a list of datasource rule query objects
            recipient (any): Specify the recipient datasource id of the alerts (default grafana)

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns the result of the specified recipient rule
        """

        if (
            (
                (type(recipient) == int and recipient != 0)
                or (type(recipient) == str and len(recipient) != 0)
            )
            and len(expr) != 0
            and len(condition) != 0
            and data_query != list()
        ):
            datasource_rule_query_objects_json: list = list()
            datasource_rule_query_object_json: dict = dict()

            for datasource_rule_query_object in data_query:
                datasource_rule_query_object_json[
                    "datasourceUid"
                ] = datasource_rule_query_object.datasource_uid
                datasource_rule_query_object_json[
                    "model"
                ] = datasource_rule_query_object.model
                datasource_rule_query_object_json[
                    "queryType"
                ] = datasource_rule_query_object.query_type
                datasource_rule_query_object_json[
                    "refId"
                ] = datasource_rule_query_object.ref_id
                datasource_rule_query_object_json[
                    "relativeTimeRange"
                ] = datasource_rule_query_object.relative_time_range
                datasource_rule_query_objects_json.append(
                    datasource_rule_query_object_json
                )

            api_call: dict = (
                Api(self.grafana_api_model)
                .call_the_api(
                    f"/api/v1/rule/test/{recipient}",
                    RequestsMethods.POST,
                    json.dumps(
                        {
                            "expr": expr,
                            "grafana_condition": {
                                "condition": condition,
                                "data": datasource_rule_query_objects_json,
                                "now": str(datetime.datetime.now()),
                            },
                        }
                    ),
                )
                .json()
            )

            if api_call == dict() or api_call.get("message") is not None:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                return api_call
        else:
            logging.error(
                "There is no recipient, expr, condition or data_query defined."
            )
            raise ValueError

    def delete_ngalert_organization_configuration(self):
        """The method includes a functionality to delete the NGAlert organization admin configuration

        Raises:
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        api_call: dict = (
            Api(self.grafana_api_model)
            .call_the_api(
                f"{APIEndpoints.ALERTS_NGALERT.value}/admin_config",
                RequestsMethods.DELETE,
            )
            .json()
        )

        if api_call != dict() and api_call.get("message") is not None:
            logging.error(f"Check the error: {api_call}.")
            raise Exception
        else:
            logging.info(
                "You successfully deleted the NGAlert organization configuration."
            )

    def get_ngalert_organization_configuration(self) -> dict:
        """The method includes a functionality to get the NGAlert organization admin configuration

        Raises:
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns the NGAlert organization configuration
        """

        api_call: any = Api(self.grafana_api_model).call_the_api(
            f"{APIEndpoints.ALERTS_NGALERT.value}/admin_config",
        )

        status_code: int = api_call.status_code

        alert_manager_status_dict: dict = dict(
            {
                404: "There's no configuration available.",
                500: "Failure, an unexpected error occurred.",
            }
        )

        if status_code == 200:
            return api_call.json()
        elif 404 <= status_code <= 500:
            logging.error(alert_manager_status_dict.get(status_code))
            raise Exception
        else:
            logging.error(f"Check the error: {api_call}.")
            raise Exception

    def create_or_update_ngalert_organization_configuration(
        self, alert_managers: list, alertmanagers_choice: str = "all"
    ):
        """The method includes a functionality to create or update the NGAlert organization admin configuration

        Args:
            alert_managers (list): Specify the list of alert manager names
            alertmanagers_choice (str): Specify the Alertmanagers choice (default all)

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if alert_managers != list() and len(alertmanagers_choice) != 0:
            api_call: dict = (
                Api(self.grafana_api_model)
                .call_the_api(
                    f"{APIEndpoints.ALERTS_NGALERT.value}/admin_config",
                    RequestsMethods.POST,
                    json.dumps(
                        {
                            "Alertmanagers": alert_managers,
                            "alertmanagersChoice": alertmanagers_choice,
                        }
                    ),
                )
                .json()
            )

            if api_call != dict():
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                logging.info(
                    "You successfully created an NGAlert organization configuration."
                )
        else:
            logging.error(
                "There is no recipient, expr, condition or data_query defined."
            )
            raise ValueError

    def get_ngalert_alertmanagers_by_organization(self) -> dict:
        """The method includes a functionality to get the discovered and dropped Alertmanagers of the user's organization and based on the specified configuration

        Raises:
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns the NGAlert Alertmanagers
        """

        api_call: dict = (
            Api(self.grafana_api_model)
            .call_the_api(
                f"{APIEndpoints.ALERTS_NGALERT.value}/Alertmanagers",
            )
            .json()
        )

        if api_call == dict():
            logging.error(f"Check the error: {api_call}.")
            raise Exception
        else:
            return api_call
